@@ -2,7 +2,7 @@ use core::fmt::{self, Debug, Formatter};
 use core::intrinsics::assume;
 use core::iter::{FusedIterator, TrustedLen};
 use core::marker::{PhantomData, Send, Sync};
-use core::mem::{size_of, MaybeUninit};
+use core::mem::{replace, size_of, MaybeUninit};
 use core::ptr;
 
 use crate::utils::{distance_between, slice_from_raw_parts, slice_from_raw_parts_mut};
@@ -14,6 +14,10 @@ use alloc::string::String;
 #[cfg(feature = "std")]
 use alloc::format;
 
+// Note that all of the iterators in this file having a constant generic `N` parameter is not really
+// useful currently, but done in a forward-thinking sense for such a time when it *is* useful in
+// order to avoid unsolvable backwards-compatibility issues at that point.
+
 /// Similar to [`Iter`](core::slice::Iter), but specifically implemented with [`StaticVec`]s in
 /// mind.
 pub struct StaticVecIterConst<'a, T: 'a, const N: usize> {
@@ -91,7 +95,11 @@ impl<'a, T: 'a, const N: usize> StaticVecIterConst<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
+// Note that the `const` iterator implementations below are `const` moreso just as groundwork for a
+// future where `for` loops are allowed in `const` contexts, and do not necessarily benefit in any
+// particularly useful way from being `const` impls quite yet.
+
+impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterConst<'a, T, N> {
   type Item = &'a T;
 
   #[inline(always)]
@@ -131,7 +139,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
 
   #[inline(always)]
   fn nth(&mut self, n: usize) -> Option<&'a T> {
-    if n >= self.len() {
+    if n >= distance_between(self.end, self.start) {
       None
     } else {
       unsafe {
@@ -157,7 +165,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const DoubleEndedIterator for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<&'a T> {
     unsafe {
@@ -183,7 +191,7 @@ impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterConst<'a, T
 
   #[inline(always)]
   fn nth_back(&mut self, n: usize) -> Option<&'a T> {
-    if n >= self.len() {
+    if n >= distance_between(self.end, self.start) {
       None
     } else {
       unsafe {
@@ -197,7 +205,7 @@ impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterConst<'a, T
   }
 }
 
-impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const ExactSizeIterator for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn len(&self) -> usize {
     distance_between(self.end, self.start)
@@ -209,12 +217,12 @@ impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecIterConst<'a, T,
   }
 }
 
-impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecIterConst<'a, T, N> {}
-unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecIterConst<'a, T, N> {}
-unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecIterConst<'a, T, N> {}
-unsafe impl<'a, T: 'a + Sync, const N: usize> Send for StaticVecIterConst<'a, T, N> {}
+impl<'a, T: 'a, const N: usize> const FusedIterator for StaticVecIterConst<'a, T, N> {}
+unsafe impl<'a, T: 'a, const N: usize> const TrustedLen for StaticVecIterConst<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecIterConst<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> const Send for StaticVecIterConst<'a, T, N> {}
 
-impl<'a, T: 'a, const N: usize> Clone for StaticVecIterConst<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const Clone for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn clone(&self) -> Self {
     Self {
@@ -267,7 +275,7 @@ impl<'a, T: 'a, const N: usize> StaticVecIterMut<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const Iterator for StaticVecIterMut<'a, T, N> {
   type Item = &'a mut T;
 
   #[inline(always)]
@@ -307,7 +315,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
 
   #[inline(always)]
   fn nth(&mut self, n: usize) -> Option<&'a mut T> {
-    if n >= self.len() {
+    if n >= distance_between(self.end, self.start) {
       None
     } else {
       unsafe {
@@ -333,7 +341,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterMut<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const DoubleEndedIterator for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<&'a mut T> {
     unsafe {
@@ -359,7 +367,7 @@ impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterMut<'a, T,
 
   #[inline(always)]
   fn nth_back(&mut self, n: usize) -> Option<&'a mut T> {
-    if n >= self.len() {
+    if n >= distance_between(self.end, self.start) {
       None
     } else {
       unsafe {
@@ -373,7 +381,7 @@ impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterMut<'a, T,
   }
 }
 
-impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecIterMut<'a, T, N> {
+impl<'a, T: 'a, const N: usize> const ExactSizeIterator for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
   fn len(&self) -> usize {
     distance_between(self.end, self.start)
@@ -385,10 +393,10 @@ impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecIterMut<'a, T, N>
   }
 }
 
-impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecIterMut<'a, T, N> {}
-unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecIterMut<'a, T, N> {}
-unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecIterMut<'a, T, N> {}
-unsafe impl<'a, T: 'a + Send, const N: usize> Send for StaticVecIterMut<'a, T, N> {}
+impl<'a, T: 'a, const N: usize> const FusedIterator for StaticVecIterMut<'a, T, N> {}
+unsafe impl<'a, T: 'a, const N: usize> const TrustedLen for StaticVecIterMut<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecIterMut<'a, T, N> {}
+unsafe impl<'a, T: 'a + Send, const N: usize> const Send for StaticVecIterMut<'a, T, N> {}
 
 impl<'a, T: 'a + Debug, const N: usize> Debug for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
@@ -705,8 +713,7 @@ impl<T, I: Iterator<Item = T>, const N: usize> Iterator for StaticVecSplice<T, I
       0 => None,
       _ => match self.replace_with.next() {
         Some(replace_with) => unsafe {
-          let removed =
-            core::mem::replace((&mut *self.vec).get_unchecked_mut(self.start), replace_with);
+          let removed = replace((&mut *self.vec).get_unchecked_mut(self.start), replace_with);
           self.start += 1;
           Some(removed)
         },
@@ -742,7 +749,7 @@ impl<T, I: Iterator<Item = T> + DoubleEndedIterator, const N: usize> DoubleEnded
       0 => None,
       _ => match self.replace_with.next_back() {
         Some(replace_with) => unsafe {
-          let removed = core::mem::replace(
+          let removed = replace(
             (&mut *self.vec).get_unchecked_mut(self.end - 1),
             replace_with,
           );
