@@ -37,6 +37,10 @@ use serde::{
   Deserialize, Deserializer, Serialize, Serializer,
 };
 
+// Note that the constness of many of the trait impls in this file varies in how useful it actually
+// is currently, and is done mostly just out of the desire to be able to quickly "stay on top" of
+// future developments as far as `const_trait_impl` is concerned.
+
 impl<T, const N: usize> const AsMut<[T]> for StaticVec<T, N> {
   #[inline(always)]
   fn as_mut(&mut self) -> &mut [T] {
@@ -113,7 +117,9 @@ impl<T: Copy, const N: usize> const Clone for StaticVec<T, N> {
         data: {
           let mut res = Self::new_data_uninit();
           unsafe {
-            ptr::copy_nonoverlapping(self.as_ptr(), Self::first_ptr_mut(&mut res), length);
+            self
+              .as_ptr()
+              .copy_to_nonoverlapping(Self::first_ptr_mut(&mut res), length);
             res
           }
         },
@@ -126,7 +132,9 @@ impl<T: Copy, const N: usize> const Clone for StaticVec<T, N> {
   fn clone_from(&mut self, rhs: &Self) {
     // Similar to what we do above, but more straightforward since `clone_from` works in-place.
     unsafe {
-      ptr::copy_nonoverlapping(rhs.as_ptr(), self.as_mut_ptr(), rhs.length);
+      self
+        .as_mut_ptr()
+        .copy_from_nonoverlapping(rhs.as_ptr(), rhs.length);
       self.set_len(rhs.length);
     }
   }
@@ -165,6 +173,7 @@ impl<T, const N: usize> const DerefMut for StaticVec<T, N> {
 impl<T, const N: usize> Drop for StaticVec<T, N> {
   #[inline(always)]
   fn drop(&mut self) {
+    // `self.as_mut_slice()` will always return a slice of known-initialized elements.
     unsafe { ptr::drop_in_place(self.as_mut_slice()) };
   }
 }
@@ -240,7 +249,7 @@ impl<T, const N1: usize, const N2: usize> ExtendEx<T, StaticVec<T, N1>> for Stat
   }
 }
 
-impl<'a, T: 'a + Copy, const N1: usize, const N2: usize> ExtendEx<&'a T, &StaticVec<T, N2>>
+impl<'a, T: 'a + Copy, const N1: usize, const N2: usize> const ExtendEx<&'a T, &StaticVec<T, N2>>
   for StaticVec<T, N1>
 {
   #[inline(always)]
@@ -254,7 +263,7 @@ impl<'a, T: 'a + Copy, const N1: usize, const N2: usize> ExtendEx<&'a T, &Static
   }
 }
 
-impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, &StaticVec<T, N>> for StaticVec<T, N> {
+impl<'a, T: 'a + Copy, const N: usize> const ExtendEx<&'a T, &StaticVec<T, N>> for StaticVec<T, N> {
   #[inline(always)]
   fn extend_ex(&mut self, iter: &StaticVec<T, N>) {
     self.extend_from_slice(iter);
@@ -266,7 +275,7 @@ impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, &StaticVec<T, N>> for Sta
   }
 }
 
-impl<'a, T: 'a + Copy, const N1: usize, const N2: usize>
+impl<'a, T: 'a + Copy, const N1: usize, const N2: usize> const
   ExtendEx<&'a T, StaticVecIterConst<'a, T, N2>> for StaticVec<T, N1>
 {
   #[inline(always)]
@@ -280,7 +289,7 @@ impl<'a, T: 'a + Copy, const N1: usize, const N2: usize>
   }
 }
 
-impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, StaticVecIterConst<'a, T, N>>
+impl<'a, T: 'a + Copy, const N: usize> const ExtendEx<&'a T, StaticVecIterConst<'a, T, N>>
   for StaticVec<T, N>
 {
   #[inline(always)]
@@ -308,7 +317,7 @@ impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, core::slice::Iter<'a, T>>
   }
 }
 
-impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, &'a [T]> for StaticVec<T, N> {
+impl<'a, T: 'a + Copy, const N: usize> const ExtendEx<&'a T, &'a [T]> for StaticVec<T, N> {
   #[inline(always)]
   fn extend_ex(&mut self, iter: &'a [T]) {
     self.extend_from_slice(iter);
@@ -334,7 +343,7 @@ impl<'a, T: 'a + Copy, const N: usize> Extend<&'a T> for StaticVec<T, N> {
   }
 }
 
-impl<T: Copy, const N: usize> From<&[T]> for StaticVec<T, N> {
+impl<T: Copy, const N: usize> const From<&[T]> for StaticVec<T, N> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -343,7 +352,7 @@ impl<T: Copy, const N: usize> From<&[T]> for StaticVec<T, N> {
   }
 }
 
-impl<T: Copy, const N: usize> From<&mut [T]> for StaticVec<T, N> {
+impl<T: Copy, const N: usize> const From<&mut [T]> for StaticVec<T, N> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -368,7 +377,7 @@ impl<T, const N: usize> const From<[T; N]> for StaticVec<T, N> {
   }
 }
 
-impl<T: Copy, const N1: usize, const N2: usize> From<&[T; N1]> for StaticVec<T, N2> {
+impl<T: Copy, const N1: usize, const N2: usize> const From<&[T; N1]> for StaticVec<T, N2> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -377,7 +386,7 @@ impl<T: Copy, const N1: usize, const N2: usize> From<&[T; N1]> for StaticVec<T,
   }
 }
 
-impl<T: Copy, const N: usize> From<&[T; N]> for StaticVec<T, N> {
+impl<T: Copy, const N: usize> const From<&[T; N]> for StaticVec<T, N> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -386,7 +395,7 @@ impl<T: Copy, const N: usize> From<&[T; N]> for StaticVec<T, N> {
   }
 }
 
-impl<T: Copy, const N1: usize, const N2: usize> From<&mut [T; N1]> for StaticVec<T, N2> {
+impl<T: Copy, const N1: usize, const N2: usize> const From<&mut [T; N1]> for StaticVec<T, N2> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -395,7 +404,7 @@ impl<T: Copy, const N1: usize, const N2: usize> From<&mut [T; N1]> for StaticVec
   }
 }
 
-impl<T: Copy, const N: usize> From<&mut [T; N]> for StaticVec<T, N> {
+impl<T: Copy, const N: usize> const From<&mut [T; N]> for StaticVec<T, N> {
   /// Creates a new StaticVec instance from the contents of `values`, using
   /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
@@ -467,44 +476,60 @@ impl<T: Hash, const N: usize> Hash for StaticVec<T, N> {
 // deferring to `SliceIndex` for it for a while it proved to to be somewhat
 // less performant due to the added indirection.
 
-impl<T, const N: usize> Index<usize> for StaticVec<T, N> {
+impl<T, const N: usize> const Index<usize> for StaticVec<T, N> {
   type Output = T;
   /// Asserts that `index` is less than the current length of the StaticVec,
   /// and if so returns the value at that position as a constant reference.
   #[inline(always)]
   fn index(&self, index: usize) -> &Self::Output {
+    // The formatted assertion macros are not const-compatible yet.
+    /*
     assert!(
       index < self.length,
       "In StaticVec::index, provided index {} must be less than the current length of {}!",
       index,
       self.length
     );
+    */
+    assert!(
+      index < self.length,
+      "Bounds check failure in StaticVec::index!",
+    );
     unsafe { self.get_unchecked(index) }
   }
 }
 
-impl<T, const N: usize> IndexMut<usize> for StaticVec<T, N> {
+impl<T, const N: usize> const IndexMut<usize> for StaticVec<T, N> {
   /// Asserts that `index` is less than the current length of the StaticVec,
   /// and if so returns the value at that position as a mutable reference.
   #[inline(always)]
   fn index_mut(&mut self, index: usize) -> &mut Self::Output {
+    // The formatted assertion macros are not const-compatible yet.
+    /*
     assert!(
       index < self.length,
       "In StaticVec::index_mut, provided index {} must be less than the current length of {}!",
       index,
       self.length
     );
+    */
+    assert!(
+      index < self.length,
+      "Bounds check failure in StaticVec::index_mut!",
+    );
     unsafe { self.get_unchecked_mut(index) }
   }
 }
 
-impl<T, const N: usize> Index<Range<usize>> for StaticVec<T, N> {
+impl<T, const N: usize> const Index<Range<usize>> for StaticVec<T, N> {
   type Output = [T];
-  /// Asserts that the lower bound of `index` is less than its upper bound,
+  /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
   /// and that its upper bound is less than or equal to the current length of the StaticVec,
   /// and if so returns a constant reference to a slice of elements `index.start..index.end`.
+  #[allow(clippy::suspicious_operation_groupings)]
   #[inline(always)]
   fn index(&self, index: Range<usize>) -> &Self::Output {
+    // This is the part that confuses Clippy.
     assert!(index.start <= index.end && index.end <= self.length);
     slice_from_raw_parts(
       unsafe { self.ptr_at_unchecked(index.start) },
@@ -513,12 +538,14 @@ impl<T, const N: usize> Index<Range<usize>> for StaticVec<T, N> {
   }
 }
 
-impl<T, const N: usize> IndexMut<Range<usize>> for StaticVec<T, N> {
-  /// Asserts that the lower bound of `index` is less than its upper bound,
+impl<T, const N: usize> const IndexMut<Range<usize>> for StaticVec<T, N> {
+  /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
   /// and that its upper bound is less than or equal to the current length of the StaticVec,
   /// and if so returns a mutable reference to a slice of elements `index.start..index.end`.
+  #[allow(clippy::suspicious_operation_groupings)]
   #[inline(always)]
   fn index_mut(&mut self, index: Range<usize>) -> &mut Self::Output {
+    // This is the part that confuses Clippy.
     assert!(index.start <= index.end && index.end <= self.length);
     slice_from_raw_parts_mut(
       unsafe { self.mut_ptr_at_unchecked(index.start) },
@@ -527,7 +554,7 @@ impl<T, const N: usize> IndexMut<Range<usize>> for StaticVec<T, N> {
   }
 }
 
-impl<T, const N: usize> Index<RangeFrom<usize>> for StaticVec<T, N> {
+impl<T, const N: usize> const Index<RangeFrom<usize>> for StaticVec<T, N> {
   type Output = [T];
   /// Asserts that the lower bound of `index` is less than or equal to the
   /// current length of the StaticVec, and if so returns a constant reference
@@ -542,7 +569,7 @@ impl<T, const N: usize> Index<RangeFrom<usize>> for StaticVec<T, N> {
   }
 }
 
-impl<T, const N: usize> IndexMut<RangeFrom<usize>> for StaticVec<T, N> {
+impl<T, const N: usize> const IndexMut<RangeFrom<usize>> for StaticVec<T, N> {
   /// Asserts that the lower bound of `index` is less than or equal to the
   /// current length of the StaticVec, and if so returns a mutable reference
   /// to a slice of elements `index.start()..self.length`.
@@ -651,6 +678,7 @@ impl<T, const N: usize> const IndexMut<RangeToInclusive<usize>> for StaticVec<T,
 }
 
 /// **Note:** this is only available when the `std` crate feature is enabled.
+#[allow(clippy::from_over_into)]
 #[cfg(feature = "std")]
 impl<T, const N: usize> Into<Vec<T>> for StaticVec<T, N> {
   /// Functionally equivalent to [`into_vec`](crate::StaticVec::into_vec).
@@ -660,7 +688,7 @@ impl<T, const N: usize> Into<Vec<T>> for StaticVec<T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> IntoIterator for &'a StaticVec<T, N> {
+impl<'a, T: 'a, const N: usize> const IntoIterator for &'a StaticVec<T, N> {
   type IntoIter = StaticVecIterConst<'a, T, N>;
   type Item = &'a T;
   /// Returns a [`StaticVecIterConst`](crate::iterators::StaticVecIterConst) over the StaticVec's
@@ -671,7 +699,7 @@ impl<'a, T: 'a, const N: usize> IntoIterator for &'a StaticVec<T, N> {
   }
 }
 
-impl<'a, T: 'a, const N: usize> IntoIterator for &'a mut StaticVec<T, N> {
+impl<'a, T: 'a, const N: usize> const IntoIterator for &'a mut StaticVec<T, N> {
   type IntoIter = StaticVecIterMut<'a, T, N>;
   type Item = &'a mut T;
   /// Returns a [`StaticVecIterMut`](crate::iterators::StaticVecIterMut) over the StaticVec's
